#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

int iterate_components(const char *input, char *dest, int idx)
{
    const char *slash;
    const char *strp;
    int len;

    assert(input[0] == '/');

    if (idx == 0) {
	*dest++ = '/';
	*dest = 0;
	return 1;
    }

    strp = input + idx;
    if (*strp == '\0') {
	return -1;
    }

    while (*strp == '/') {
	++strp;
    }
    slash = strchr(strp, '/');
    if (slash == NULL) {
	slash = strp + strlen(strp);
    }
    len = slash - (input + idx);
    assert(len > 0);	/* we are making forward progress */
    memcpy(dest + idx, input + idx, len);
    idx += len;
    dest[idx] = 0;
    return idx;
}


typedef struct {
    const char *input;
    const char *outputs[10];
} a_test_input;

const a_test_input data[] = {
    { "/",		{ "/", NULL } },
    { "/a",		{ "/", "/a", NULL } },
    { "/ab/c",		{ "/", "/ab", "/ab/c", NULL } },
    { "/a/cd",		{ "/", "/a", "/a/cd", NULL } },
    { "/abc/d/ef",	{ "/", "/abc", "/abc/d", "/abc/d/ef", NULL } },
    { "/abc/d/ef/",	{ "/", "/abc", "/abc/d", "/abc/d/ef", "/abc/d/ef/", NULL } },
    { "//a",		{ "/", "//a", NULL } },
    { "//a//b",		{ "/", "//a", "//a//b", NULL } },
    { "//ab//b",	{ "/", "//ab", "//ab//b", NULL } },
    { "//a//bc",	{ "/", "//a", "//a//bc", NULL } },
    { "//a//bc//",	{ "/", "//a", "//a//bc", "//a//bc//", NULL } },
    { "//aa//bc///def",	{ "/", "//aa", "//aa//bc", "//aa//bc///def", NULL } }
};

int main(int argc, char **argv)
{
    int i;
    int j;
    char input[512];
    char output[512];
    int old_idx;
    int idx;
    int err;

    err = 0;
    for (i = 0; i < sizeof(data)/sizeof(data[0]); ++i) {
	j = 0;
	old_idx = 0;
	strcpy(input, data[i].input);
	for (;;) {
	    idx = iterate_components(input, output, old_idx);
	    if (idx == -1) break;
	    if (data[i].outputs[j] == NULL) {
		printf("error: input = \"%s\", old_idx = %d, output = \"%s\", did not expect component %d\n",
		    input, old_idx, output, j);
		err = 1;
		break;
	    }
	    if (strcmp(output, data[i].outputs[j])) {
		printf("error: input = \"%s\", old_idx = %d, output = \"%s\", expected = \"%s\"\n",
		    input, old_idx, output, data[i].outputs[j]);
		err = 1;
	    }
	    if (idx != strlen(output)) {
		printf("error: input = \"%s\", old_idx = %d, output = \"%s\", idx = %d, idx != strlen(output)\n",
		    input, old_idx, output, idx);
		err = 1;
	    }
	    ++j;
	    old_idx = idx;
	}
	if (data[i].outputs[j] != NULL) {
	    printf("error: input = \"%s\", expected more than %d components\n",
		input, j);
	    err = 1;
	}
    }
    return err;
}
